Struct qwt::bitvector::BitVector

source ·
pub struct BitVector { /* private fields */ }

Implementations

source

impl BitVector

source

pub fn new() -> Self

Creates a new empty binary vector.

source

pub fn with_capacity(n_bits: usize) -> Self

Creates an empty binary vector with at least a capacity of n_bits.

source

pub fn with_zeros(n_bits: usize) -> Self

Creates a binary vector with n_bits set to 0.

source

pub fn push(&mut self, bit: bool)

Pushes a bit at the end of the binary vector.

Example
use qwt::{BitVector, AccessBin};

let mut bv = BitVector::new();
bv.push(true);
bv.push(false);
bv.push(true);
assert_eq!(bv.len(), 3);
assert_eq!(bv.get(0), Some(true));
source

pub fn append_bits(&mut self, bits: u64, len: usize)

Appends len bits at the end of the bit vector by taking the least significant len bits in the u64 value bits.

Panics

Panics if len is larger than 64 or if a bit of position
larger than len is set in bits.

Examples
use qwt::BitVector;

let mut bv = BitVector::new();
bv.append_bits(5, 3); // appends 101
bv.append_bits(6, 4); // appends 0110          
assert_eq!(bv.len(), 7);
assert_eq!(bv.get_bits(0, 3), Some(5));
source

pub fn extend_with_zeros(&mut self, n: usize)

Extends the bit vector by adding n bits set to 0.

Examples
use qwt::{BitVector, AccessBin};

let mut bv = BitVector::new();
bv.extend_with_zeros(10);
assert_eq!(bv.len(), 10);
assert_eq!(bv.get(8), Some(false));
source

pub fn set(&mut self, index: usize, bit: bool)

Sets the to bit the given position index.

Panics

Panics if index is out of bounds.

source

pub fn get_bits(&self, index: usize, len: usize) -> Option<u64>

Accesses len bits, with 1 <= len <= 64, starting at position index.

Returns None if index+len is out of bounds or len == 0 or len > 64.

source

pub fn set_bits(&mut self, index: usize, len: usize, bits: u64)

Sets len bits, with 1 <= len <= 64, starting at position index to the len least significant bits in bits.

Panics

Panics if index+len is out of bounds or len > 64 or if most significant bit in bits ia at a position larger than of equal to len.

source

pub fn ones(&self) -> BitVectorBitPositionsIter<'_, true>

Returns an iterator over positions of bit set to 1 in the bit vector.

Examples
use qwt::BitVector;
let vv: Vec<usize> = vec![0, 63, 128, 129, 254, 1026];
let bv: BitVector = vv.iter().copied().collect();

let v: Vec<usize> = bv.ones().collect();
assert_eq!(v, vv);
source

pub fn zeros(&self) -> BitVectorBitPositionsIter<'_, false>

Gives an iterator over positions of bit set to 0 in the bit vector.

source

pub fn get_word(&self, i: usize) -> u64

Gets a whole 64bit word.

source

pub fn shrink_to_fit(&mut self)

Shrinks the underlying vector of 64bit words to fit.

source

pub fn is_empty(&self) -> bool

Checks if the vector is empty.

source

pub fn len(&self) -> usize

Returns the number of bits in the bitvector.

source

pub fn iter(&self) -> BitVectorIter<'_>

Returns a non-consuming iterator over bits of the bit vector.